"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.$button = exports.$form = exports.$input = exports.$textarea = exports.$div = exports.$span = exports.$p = exports.$img = exports.$a = exports.createElement$ = void 0;
var react_1 = require("react");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var fragment_1 = require("./fragment");
// TODO: handle ref
// TODO: add better TS support
var EMPTY_DEPENDENCIES = [];
var useEmptyObject = function () { return react_1.useMemo(function () { return Object.create(null); }, EMPTY_DEPENDENCIES); };
/**
 * creates a dynamic component that will subscribe to passed Observable props
 *
 * Example:
 *
 * ```jsx
 * import React from 'react';
 * import { createElement$ } from 'react-rxjs-elements';
 * import { timer } from 'rxjs';
 *
 * function App(){
 *   const $div = createElement$('div');
 *   const timer$ = timer(0, 1000);
 *
 *   return <$div title={ timer$ } >Hello world</div>
 * }
 * ```
 *
 * @param element element tag ('div', 'input', '...') or component function/class
 * @returns FunctionComponent that observes it's Observable params and children
 */
function createElement$(element) {
    return function (props) {
        // state for renderable props
        var _a = react_1.useState(Object.create(null)), stateProps = _a[0], setStateProps = _a[1];
        // placeholder for Observable.error case -- will use it to rethrow error
        var _b = react_1.useState(null), error = _b[0], setError = _b[1];
        // destroy$ stream helper {{{
        // it will emit an empty value on unmount
        var destroy$ = react_1.useMemo(function () { return new rxjs_1.Subject(); }, EMPTY_DEPENDENCIES);
        react_1.useEffect(function () { return function () {
            destroy$.next(void 0);
            destroy$.complete();
        }; }, EMPTY_DEPENDENCIES);
        // }}}
        // store prev props to compare
        var _prevProps = useEmptyObject();
        // keep subscriptions to unsubscribe on dynamic prop update
        var _subs = useEmptyObject();
        // useMemo to update props
        // unlike useEffect, it will be executed with the first render
        react_1.useMemo(function () {
            // check for obsolete props
            var delProps = Object.create(null);
            Object.keys(_prevProps).forEach(function (key) {
                if (key in props) {
                    return;
                }
                delete _prevProps[key];
                // if previous property was Observable
                // kill subscription
                cleanSubscription(_subs, key);
                // remove from static props
                delProps[key] = void 0;
            });
            // update/track new props
            var nextProps = Object.create(null);
            var streamKeys = [];
            Object.keys(props).forEach(function (key) {
                // children are covered via <$>
                if (key == 'children') {
                    return;
                }
                // if property changes and previous was Observable
                // we need to kill subscription
                if (!Object.is(props[key], _prevProps[key])) {
                    cleanSubscription(_subs, key);
                }
                var value = props[key];
                _prevProps[key] = value;
                // observable input params are auto observed
                if (rxjs_1.isObservable(value)) {
                    streamKeys.push(key);
                }
                // all static props are directly updated
                else {
                    nextProps[key] = value;
                }
            });
            // remove obsolete props
            // & update static props
            setStateProps(function (p) { return Object.assign({}, p, delProps, nextProps); });
            streamKeys.map(function (key) {
                _subs[key] = props[key]
                    .pipe(operators_1.takeUntil(destroy$))
                    .subscribe({
                    // on value updates -- update props
                    next: function (data) {
                        setStateProps(function (p) {
                            var _a;
                            return Object.assign({}, p, (_a = {}, _a[key] = data, _a));
                        });
                    },
                    // on error -- rethrow error
                    error: function (error) {
                        setError({ error: error });
                    }
                });
            });
        }, [props]);
        // if error -- throw
        if (error) {
            throw error.error;
        }
        return react_1.createElement(element, stateProps, 
        // if children exist
        // they might be observable
        // so we pass em to <$> fragment
        // NOTE: children might not exist for elements like <input />
        props.children
            ? react_1.createElement(fragment_1.$, null, props.children)
            : null);
    };
}
exports.createElement$ = createElement$;
// helpers
function cleanSubscription(store, key) {
    if (store[key]) {
        store[key].unsubscribe();
        delete store[key];
    }
}
// preset elements {{{
// Q: should we prefab all possible elements?
// Q: should that be statically generated on build or created on demand?
exports.$a = createElement$('a');
exports.$img = createElement$('img');
exports.$p = createElement$('p');
exports.$span = createElement$('span');
exports.$div = createElement$('div');
exports.$textarea = createElement$('textarea');
exports.$input = createElement$('input');
exports.$form = createElement$('form');
exports.$button = createElement$('button');
// }}}
