var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
import { createElement, Fragment, useEffect, useState } from "react";
import { isObservable } from "rxjs";
// TODO: use useMemo instead of useEffect
// TODO: add better TS support
/**
 * <$> fragment will subscribe to it's Observable children and display
 * it's emissions along with regular children
 *
 * e.g.
 *
 * ```jsx
 * function App(){
 *   return <$>{ timer(0, 1000) }</$>    // 0, 1, 2, 3, ...
 * }
 * ```
 */
export function $(props) {
    var children = props === null || props === void 0 ? void 0 : props.children;
    // store children
    var _a = useState(null), cn = _a[0], setCn = _a[1];
    var _b = useState(null), error = _b[0], setError = _b[1];
    useEffect(function () {
        // multiple children, one of em might be observable
        if (Array.isArray(children)) {
            setCn(children.map(function (c) { return createElement($, null, c); }));
            return;
        }
        // single observable child
        if (isObservable(children)) {
            var sub_1 = children.subscribe({
                next: function (cn) {
                    // update the view
                    setCn(cn);
                },
                error: function (error) {
                    // wrap error in an object to be safe in case the error is nullish
                    setError({ error: error });
                }
            });
            return function () { return sub_1.unsubscribe(); };
        }
        // child is just a child
        setCn(children);
    }, [children]);
    // raise an error if Observable failed
    if (error) {
        throw error.error;
    }
    // render children as array
    return Array.isArray(cn)
        ? createElement.apply(void 0, __spreadArrays([Fragment, null], cn)) : createElement(Fragment, null, cn); // TODO: investigate if Fragment is needed here
}
